% Recursive versions of the Levenberg-Marquardt reassigned spectrogram 
% and of the synchrosqueezed STFT short demo
%
% Recommended requirements: 
% -TFTB (http://tftb.nongnu.org/index_fr.html)
% -ghostscript (http://ghostscript.com/download/ ps2pdf)
%
% Author: D. Fourer (dominique@fourer.fr)
% Date: Sept 2015
% Ref: [D.Fourer, F. Auger and P.Flandrin. Recursive versions of the Levenberg-Marquardt
% reassigned spectrogram and of the synchrosqueezed STFT. IEEE Proc. ICASSP 2016.]

clear all
close all

alpha=0.5;             %% TFR compression factor (modify display contrast)

N_0pad = 50;           %% number of zeros to add before signal (used for reconstruction)
AddedZeros = zeros(N_0pad, 1);


%% input signal
s = real([AddedZeros;fmconst(60,0.1);fmconst(60,0.2);fmconst(60,0.3);zeros(40,1)]);
N = 180+40;         %% signal length (3*60+40)
t = 1:length(s);

figure(1);
plot(s);


%% TF analysis
k=3;                   %% recursive filter order
L=10;                  %% analysis window length
mi=1; mf=250; M=500;   %% frequency range

[tfr, nfreqs] = recursive_stft(s, k, L, mi, mf, M);
[~, rtfr, nfreqs, before, after, below, over] = recursive_rsp(s, k, L, mi, mf, M);

%% Recursive spectrogram
figure(2);
imagesc(t, nfreqs, (abs(tfr).^2).^alpha);
set(gca,'YDir','normal')
colormap gray;
colormap(flipud(colormap)); 
xlabel('time samples', 'FontSize', 16)
ylabel('normalized frequency', 'FontSize', 16)
title(sprintf('Recursive spectrogram k=%d, L=%2.2f ', k, L), 'FontSize', 14);
% saveas(gcf, 'recursive_spectrogram.eps', chemin0, index), 'epsc');

%% Recursive reassigned spectrogram
figure(3);
imagesc(t, nfreqs, rtfr.^alpha);
set(gca,'YDir','normal')
colormap gray;
colormap(flipud(colormap)); 
xlabel('time samples', 'FontSize', 16)  %, 'FontName', 'Times-Roman', 'FontSize', 20
ylabel('normalized frequency', 'FontSize', 16)
title(sprintf('Recursive reassigned spectrogram k=%d, L=%2.2f ', k, L), 'FontSize', 14);
% saveas(gcf, sprintf('%s/rrspectrum_%d.eps', chemin1, index2), 'epsc');

%% Recursive synchrosqueezed STFT
n0 = round((k-1) * L);
[~, stfr, nfreqs, below, over] = recursive_sstft(s, k, L, mi, mf, M, n0);
figure(4);
imagesc(t, nfreqs, (abs(stfr).^2).^alpha);
set(gca,'YDir','normal')
colormap gray;
colormap(flipud(colormap)); 
xlabel('time samples', 'FontSize', 16)  %, 'FontName', 'Times-Roman', 'FontSize', 20
ylabel('normalized frequency', 'FontSize', 16)
title(sprintf('Recursive synchrosqueezed STFT k=%d, L=%2.2f ', k, L), 'FontSize', 14);
% saveas(gcf, sprintf('%s/rrspectrum_%d.eps', chemin1, index2), 'epsc');


%% Signal reconstruction from recursive STFT
n_range     = N_0pad+(1:N);
n_range_rec = n_range-n0;
s_hat = stft_rec(tfr, k, L, mi:mf, M, n0);
s_rec = 2*real(s_hat(n_range_rec)); %% assume the signal is real
rqf_stft = RQF(s(n_range_rec), s_rec);
fprintf(1,'Recursive STFT reconstruction : RQF=%0.02f dB\n', rqf_stft);

figure(1);
plot(t(n_range_rec), s(n_range_rec), 'g-.');
hold on
plot(t(n_range_rec), s_rec, 'k');
legend('original signal', 'reconstructed signal')
title('recursive STFT reconstruction')
title(sprintf('recursive synchrosqueezed STFT reconstruction, RQF=%.2f dB', rqf_stft))


%% Signal reconstruction from recursive synchrosqueezed STFT
s_hat = sstft_rec(stfr, k, L, M, n0);
s_rec = 2*real(s_hat(n_range_rec)); %% assume the signal is real
rqf_sstft =  RQF(s(n_range_rec), s_rec);
fprintf(1,'Recursive synchrosqueezed STFT reconstruction : RQF=%0.02f dB\n', rqf_sstft);

figure(1);
plot(t(n_range_rec), s(n_range_rec), 'g-.');
hold on
plot(t(n_range_rec), s_rec, 'k');
legend('original signal', 'reconstructed signal')
title(sprintf('recursive synchrosqueezed STFT reconstruction, RQF=%.2f dB', rqf_sstft))
